SystemOrganization addCategory: #'StreamingHtmlCanvas-Core'!
SystemOrganization addCategory: #'StreamingHtmlCanvas-Tests'!

TestCase subclass: #StreamingHtmlCanvasTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Tests'!

!StreamingHtmlCanvasTest methodsFor: 'helper methods' stamp: 'ul 11/1/2011 15:59'!
should: aBlock print: aString

	| result |
	result := StreamingHtmlCanvas render: aBlock.
	self assert: aString equals: result! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:03'!
testClassAttributes

	self
		should: [ :html |
			html div class: 'foo'.
			html div
				id: '1';
				class: 'bar';
				with: 'foobar'.
			html div
				class: 'foo';
				class: 'bar'.
			html div
				class: 'foo';
				class: 'foobar';
				with: 'foobarbaz' ]
		print: '<div class="foo"></div><div id="1" class="bar">foobar</div><div class="foo bar"></div><div class="foo foobar">foobarbaz</div>'

	! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:02'!
testClassIfAttributes

	self
		should: [ :html |
			html div class: 'foo' if: false.
			html div class: 'foo' if: true.
			html div
				class: 'bar' if: true;
				class: 'baz' if: false.
			html div
				class: 'foo' if: false;
				class: 'bar' if: true;
				class: 'baz' if: true;
				with: 'barbaz' ]
		print: '<div></div><div class="foo"></div><div class="bar"></div><div class="bar baz">barbaz</div>'

	! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 1/28/2012 07:28'!
testCommentWithSibling

	self
		 should: [ :html | html span; comment: 'hello' ]
		print: '<span></span><!!--hello-->'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 1/28/2012 07:36'!
testEnsureId

	| id text |
	text := StreamingHtmlCanvas render: [ :html |
		| tag |
		tag := html div.
		id := tag ensureId.
		self assert: id equals: tag ensureId ].
	self assert: '<div id="', id, '"></div>' equals: text! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:42'!
testIntegerAsText

	self should: [ :html | html text: 42 ] print: '42'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 22:53'!
testMultipleAttributes

	self
		should: [ :html |
			html div
				id: '1';
				style: 'text-align: right';
				with: [ html text: 'foo' ] ]
		print: '<div id="1" style="text-align: right">foo</div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:02'!
testNestedSiblingTags

	self
		should: [ :html |
			html div: [
				html 
					span;
					div ] ]
		print: '<div><span></span><div></div></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:02'!
testNestedTagAddedWithWith

	self
		should: [ :html | 
			html div with: [
				html div ] ]
		print: '<div><div></div></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:38'!
testNestedTagWithText

	self
		should: [ :html | 
			html div: [
				html span: [
					html text: 'hello' ] ] ]
		print: '<div><span>hello</span></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:01'!
testNestedTags

	self
		should: [ :html | 
			html div: [
				html span ] ]
		print: '<div><span></span></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:41'!
testNilAsText

	self should: [ :html | html text: nil ] print: ''! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:07'!
testNonClosableTags

	self
		should: [ :html |
			html 
				img;
				br;
				hr;
				meta ]
		print: '<img><br><hr><meta>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:01'!
testSiblingTags

	self
		should: [ :html | html div; span; div ]
		print: '<div></div><span></span><div></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:36'!
testSimpleText

	self
		should: [ :html | html text: 'hello' ]
		print: 'hello'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:01'!
testSingleAttribute

	self
		should: [ :html |
			html div style: 'border: 1px'.
			html span
				style: 'margin: 0px';
				with: 'foo'.
			html div
				style: 'white-space: nowrap';
				with: [
					html span
						style: 'text-align: center';
						with: 'bar' ] ]
		print: '<div style="border: 1px"></div><span style="margin: 0px">foo</span><div style="white-space: nowrap"><span style="text-align: center">bar</span></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:01'!
testSingleTag

	self
		should: [ :html | html div ]
		print: '<div></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'kb 11/1/2011 11:24'!
testSingleTagWhereNoTrailingSlashIsNeeded

	self
		should: [ :html | html link href: 'style.css'; rel: 'stylesheet' ]
		print: '<link href="style.css" rel="stylesheet">'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:00'!
testSingleTagWithAttribute

	self
		should: [ :html | html div class: 'aClass' ]
		print: '<div class="aClass"></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 1/28/2012 07:37'!
testSpaceWithSibling

	self
		should: [ :html | html div; space ]
		print: '<div></div>&nbsp;'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 03:05'!
testTagWithIntegerAsText

	self should: [ :html | html div: 42 ] print: '<div>42</div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:41'!
testTagWithNilAsText

	self should: [ :html | html div: nil ] print: '<div></div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 12/30/2011 14:07'!
testTagWithNonClosableTags

	self
		should: [ :html |
			html form: [
				html input value: '1'.
				html input value: '2' ] ]
		print: '<form><input value="1"><input value="2"></form>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:37'!
testTagWithText

	self
		should: [ :html | html div: 'hello' ]
		print: '<div>hello</div>'! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'klub 5/21/2015 20:31'!
testTextMixedWithTags

	self
		should: [ :html | 
			html div.
			html text: 'bar' ]
		print: '<div></div>bar';
		should: [ :html | 
			html text: 'bar'.
			html div ]
		print: 'bar<div></div>'.! !

!StreamingHtmlCanvasTest methodsFor: 'tests' stamp: 'ul 11/1/2011 02:40'!
testTextWithEscapableCharacters

	self
		should: [ :html | html text: '<foo bar="&" baz='';''></foo>' ]
		print: '&lt;foo bar=&quot;&amp;&quot; baz=&apos;;&apos;&gt;&lt;/foo&gt;'! !

!UndefinedObject methodsFor: '*StreamingHtmlCanvas' stamp: 'ul 11/1/2011 15:51'!
renderOn: html! !

!Object methodsFor: '*StreamingHtmlCanvas' stamp: 'ul 11/1/2011 15:50'!
renderOn: html

	html text: self! !

Object subclass: #StreamingHtmlCanvas
	instanceVariableNames: 'stream stack stackIndex idGenerator scriptCollector data scriptEncoder attributeStream'
	classVariableNames: 'CharactersToEncode'
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Core'!

!StreamingHtmlCanvas class methodsFor: 'as yet unclassified' stamp: 'ul 10/31/2011 19:10'!
initialize

	CharactersToEncode := '<>&''"' asCharacterSet! !

!StreamingHtmlCanvas class methodsFor: 'as yet unclassified' stamp: 'ul 9/2/2011 00:02'!
on: stream

	^self new
		stream: stream;
		yourself! !

!StreamingHtmlCanvas class methodsFor: 'as yet unclassified' stamp: 'kb 11/1/2011 11:11'!
render: aBlock

	^String streamContents: [ :stream |
		| html |
		html := self on: stream.
		aBlock value: html.
		html flush ]! !

!StreamingHtmlCanvas class methodsFor: 'as yet unclassified' stamp: 'klub 7/14/2015 12:29'!
voidElements
	"HTML5 void elements from http://www.w3.org/TR/html5/syntax.html#void-elements"
	
	^#(area base br col embed hr img input keygen link meta param source track wbr)! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'ul 10/15/2018 00:20'!
attributeStream

	^attributeStream ifNil: [ attributeStream := (String new: 100) writeStream ]! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'klub 4/20/2015 13:16'!
close: aTag

	| currentTag |
	[
		(currentTag := stack at: stackIndex) printClosingTagOn: stream.
		stackIndex := stackIndex - 1.
		currentTag == aTag ] whileFalse! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'kb 11/29/2011 11:58'!
closeSiblings

	| tag |
	(stackIndex > 0 and: [ (tag := stack at: stackIndex) isLeaf ]) ifTrue: [
		self close: tag ]! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 1/28/2012 07:29'!
comment: aString

	self closeSiblings.
	stream nextPutAll: '<!!--'.
	self printEscapedStringForHtml: aString.
	stream nextPutAll: '-->'! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'klub 7/14/2015 13:13'!
compileAccessor: aSymbol

	| tagName numArgs |
	numArgs := aSymbol numArgs.
	tagName := (numArgs = 1
		ifTrue: [ aSymbol allButLast ]
		ifFalse: [ aSymbol ]) asString.
	self class
		compileSilently: ('{1} ^(self createTagNamed: {2} closeIfEmpty: {3}) {4}'
			format: {
				numArgs = 1
					ifTrue: [ tagName, ': argument' ]
					ifFalse: [ tagName ].
				tagName printString.
				(self class voidElements includes: tagName) not.
				numArgs = 1
					ifTrue: [ 'with: argument' ]
					ifFalse: [ '' ] })
		classified: '*generated'! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'klub 7/14/2015 13:08'!
createTagNamed: tagName closeIfEmpty: closeIfEmpty

	self closeSiblings.
	stackIndex := stackIndex + 1.
	stack size < stackIndex ifTrue: [
		^stack add: (StreamingHtmlElement
			on: self
			tag: tagName
			closeIfEmpty: closeIfEmpty) ].
	^(stack at: stackIndex)
		tag: tagName
		closeIfEmpty: closeIfEmpty! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 11/29/2012 21:23'!
dataAt: key

	^data ifNotNil: [ data at: key ifAbsent: [ nil ] ]
! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 11/29/2012 21:22'!
dataAt: key put: value

	^(data ifNil: [ data := Dictionary new ]) at: key put: value
! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'kb 11/1/2011 08:17'!
doctype401Transitional

	stream nextPutAll: '<!!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">'! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 12/30/2011 14:37'!
doctypeHtml5

	stream nextPutAll: '<!!DOCTYPE html>'! !

!StreamingHtmlCanvas methodsFor: 'error handling' stamp: 'ul 11/1/2011 04:27'!
doesNotUnderstand: aMessage

	| selector |
	((selector := aMessage selector) isInfix or: [
		selector numArgs > 1 ]) ifTrue: [ 
			^super doesNotUnderstand: aMessage ].
	self compileAccessor: selector.
	^aMessage sendTo: self! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 11/11/2014 13:53'!
encodeScript: aString

	self rawHtml: (scriptEncoder
		ifNil: [ aString ]
		ifNotNil: [ scriptEncoder value: aString ])! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'ul 11/1/2011 00:34'!
flush

	[ stackIndex > 0 ] whileTrue: [
		(stack at: stackIndex) printClosingTagOn: stream.
		stackIndex := stackIndex - 1 ]! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 4/20/2015 15:35'!
idGenerator

	^idGenerator ifNil: [ idGenerator := StreamingHtmlIdGenerator new ]! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 4/20/2015 15:36'!
idGenerator: aStreamingHtmlIdGenerator

	idGenerator := aStreamingHtmlIdGenerator! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 12/30/2011 15:25'!
if: conditionString then: aBlock

	self closeSiblings.
	stream
		nextPutAll: '<!!--[if ';
		nextPutAll: conditionString;
		nextPutAll: ']>'.
	aBlock value.
	self closeSiblings.
	stream nextPutAll: '<!![endif]-->'! !

!StreamingHtmlCanvas methodsFor: 'initialize-release' stamp: 'klub 4/20/2015 15:35'!
initialize

	stack := OrderedCollection new.
	stackIndex := 0! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'ul 10/31/2011 18:41'!
open: aTag

	aTag printOpeningTagOn: stream! !

!StreamingHtmlCanvas methodsFor: 'private' stamp: 'ul 11/1/2011 02:36'!
printEscapedStringForHtml: aString

	| start index |
	start := 1.
	(index := aString indexOfAnyOf: CharactersToEncode startingAt: start) = 0 ifTrue: [
		^stream nextPutAll: aString ].
	[ 
		stream 
			next: index - start putAll: aString startingAt: start;
			nextPutAll: (
				(aString at: index) caseOf: { 
							[ $< ] -> [ '&lt;' ].
							[ $> ] -> [ '&gt;' ].
							[ $& ] -> [ '&amp;' ].
							[ $" ] -> [ '&quot;' ].
							[ $' ] -> [ '&apos;' ] }).
		(start := index + 1) <= aString size 
			and: [ (index := aString indexOfAnyOf: CharactersToEncode startingAt: start) > 0 ] ] whileTrue.
	stream next: aString size - start + 1 putAll: aString startingAt: start! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 12/30/2011 15:19'!
rawHtml: aString

	self closeSiblings.
	stream nextPutAll: aString! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 5/9/2013 07:08'!
removeContentBetween: start and: end

	| currentPosition |
	currentPosition := stream position.
	stream position: start.
	[ stream position < end ] whileTrue: [
		stream nextPut: $  ].
	stream position: currentPosition! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'ul 11/1/2011 15:51'!
render: aRenderable

	aRenderable renderOn: self! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 11/18/2012 11:16'!
renderCollectedScripts

	scriptCollector ifNil: [ ^self ].
	scriptCollector renderOn: self! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 11/18/2012 11:15'!
scriptCollector

	^scriptCollector ifNil: [ 
		scriptCollector := StreamingHtmlScriptCollector new ]! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 11/11/2014 13:53'!
scriptEncoder: aBlock

	scriptEncoder := aBlock! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 1/28/2012 07:38'!
space

	self closeSiblings.
	stream nextPutAll: '&nbsp;'! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'ul 9/1/2011 22:14'!
stream: aWriteStream

	stream := aWriteStream! !

!StreamingHtmlCanvas methodsFor: 'accessing' stamp: 'klub 5/9/2013 06:39'!
streamPosition

	^stream position! !

!StreamingHtmlCanvas methodsFor: 'building' stamp: 'klub 5/2/2014 16:25'!
text: anObject

	(anObject isString and: [ anObject isOctetString not ]) ifTrue: [
		self error: 'Unexpected widestring: ', anObject ].
	anObject ifNotNil: [
		self closeSiblings.
		self printEscapedStringForHtml: anObject asString ]! !

Object subclass: #StreamingHtmlElement
	instanceVariableNames: 'tag attributes attributeIndex opened canvas isLeaf closeIfEmpty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Core'!

!StreamingHtmlElement class methodsFor: 'as yet unclassified' stamp: 'klub 7/14/2015 12:54'!
on: aStreamingHtmlCanvas tag: aString closeIfEmpty: aBoolean

	^self basicNew
		canvas: aStreamingHtmlCanvas
		tag: aString
		closeIfEmpty: aBoolean! !

!StreamingHtmlElement methodsFor: 'accessing' stamp: 'klub 2/9/2019 17:10'!
addAttribute: key value: aStringOrBlock

	| valueString |
	valueString := aStringOrBlock isBlock
		ifFalse: [ aStringOrBlock ]
		ifTrue: [ self evaluateAttributeValueBlock: aStringOrBlock ].
	1 to: attributeIndex do: [ :index |
		| attribute |
		(attribute := attributes at: index) key = key ifTrue: [
			attribute value: valueString.
			^self ] ].
	(attributeIndex := attributeIndex + 1) <= attributes size
		ifFalse: [ attributes add: key -> valueString ]
		ifTrue: [ 
			(attributes at: attributeIndex)
				key: key
				value: valueString ]! !

!StreamingHtmlElement methodsFor: 'initialize-release' stamp: 'klub 7/14/2015 12:51'!
canvas: aStreamingHtmlCanvas tag: aString closeIfEmpty: aBoolean

	tag := aString.
	attributes := OrderedCollection new.
	attributeIndex := 0.
	opened := false.
	canvas := aStreamingHtmlCanvas.
	isLeaf := true.
	closeIfEmpty := aBoolean! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 5/7/2013 11:43'!
checkedIf: aBoolean

	aBoolean ifTrue: [ self checked: 'checked' ]! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 10/20/2018 16:00'!
class: aStringOrBlock

	| value |
	value := aStringOrBlock isBlock
		ifFalse: [ aStringOrBlock ]
		ifTrue: [ self evaluateAttributeValueBlock: aStringOrBlock ].
	1 to: attributeIndex do: [ :index |
		| attribute |
		(attribute := attributes at: index) key = 'class' ifTrue: [
			attribute value: attribute value, ' ', value asString.
			^self ] ].
	(attributeIndex := attributeIndex + 1) <= attributes size
		ifFalse: [ attributes add: 'class' -> value ]
		ifTrue: [ 
			(attributes at: attributeIndex)
				key: 'class'
				value: value ]! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'ul 11/1/2011 20:35'!
class: aString if: aBoolean

	aBoolean ifTrue: [ self class: aString ]! !

!StreamingHtmlElement methodsFor: 'private' stamp: 'ul 11/4/2011 14:26'!
compileAccessor: aSymbol

	self class
		compileSilently: ('{1} argument self addAttribute: {2} value: argument'
			format: {
				aSymbol.
				aSymbol asString allButLast printString })
		classified: '*generated'! !

!StreamingHtmlElement methodsFor: 'accessing' stamp: 'klub 1/25/2013 08:08'!
data: key value: value

	self addAttribute: 'data-', key value: value! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 2/1/2018 11:18'!
disabledIf: aBoolean

	aBoolean ifTrue: [ self disabled: 'true' ]! !

!StreamingHtmlElement methodsFor: 'error handling' stamp: 'ul 11/1/2011 20:45'!
doesNotUnderstand: aMessage

	| selector |
	((selector := aMessage selector) isKeyword and: [
		selector numArgs = 1 ]) ifFalse: [ 
			^super doesNotUnderstand: aMessage ].
	self compileAccessor: selector.
	^aMessage sendTo: self! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 1/28/2012 07:32'!
ensureId

	| id |
	1 to: attributeIndex do: [ :index |
		| attribute |
		(attribute := attributes at: index) key = 'id' ifTrue: [	^attribute value ] ].
	(attributeIndex := attributeIndex + 1) <= attributes size
		ifFalse: [ attributes add: 'id' -> (id := canvas idGenerator nextId) ]
		ifTrue: [ 
			(attributes at: attributeIndex)
				key: 'id'
				value: (id := canvas idGenerator nextId) ].
	^id! !

!StreamingHtmlElement methodsFor: 'private' stamp: 'ul 10/15/2018 00:24'!
evaluateAttributeValueBlock: aBlock

	| stream |
	stream := canvas attributeStream.
	stream reset.
	aBlock value: stream.
	^stream contents! !

!StreamingHtmlElement methodsFor: 'accessing' stamp: 'ul 11/1/2011 16:53'!
isLeaf

	^isLeaf! !

!StreamingHtmlElement methodsFor: 'printing' stamp: 'ul 11/1/2011 16:50'!
printClosingTagOn: stream

	opened
		ifTrue: [ 
			stream
				nextPutAll: '</';
				nextPutAll: tag;
				nextPut: $> ]
		ifFalse: [ self printOpeningTagOn: stream ]! !

!StreamingHtmlElement methodsFor: 'printing' stamp: 'klub 11/3/2012 12:08'!
printOpeningTagOn: stream

	stream
		nextPut: $<;
		nextPutAll: tag.
	1 to: attributeIndex do: [ :index |
		| attribute |
		attribute := attributes at: index.
		stream
			space;
			nextPutAll: attribute key;
			nextPutAll: '="'.
		canvas printEscapedStringForHtml: attribute value asString.
		stream
			nextPut: $" ].
	stream nextPut: $>.
	opened := true.
	(closeIfEmpty and: [ isLeaf ]) ifTrue: [
		self printClosingTagOn: stream ]
! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 11/18/2012 11:15'!
script: aBlock

	aBlock value: canvas scriptCollector value: self ensureId! !

!StreamingHtmlElement methodsFor: 'building' stamp: 'klub 4/17/2013 14:40'!
selectedIf: aBoolean

	aBoolean ifTrue: [ self selected: 'selected' ]! !

!StreamingHtmlElement methodsFor: 'initialize-release' stamp: 'ul 10/14/2018 23:58'!
tag: aString closeIfEmpty: aBoolean

	tag := aString.
	attributeIndex = 0 ifFalse: [
		attributeIndex := 0.
		attributes reset ].
	opened := false.
	isLeaf := true.
	closeIfEmpty := aBoolean! !

!StreamingHtmlElement methodsFor: 'accessing' stamp: 'ul 11/1/2011 16:52'!
with: anObject

	isLeaf := false.
	canvas open: self.
	anObject renderOn: canvas.
	canvas close: self! !

Object subclass: #StreamingHtmlIdGenerator
	instanceVariableNames: 'counter limit length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Core'!

!StreamingHtmlIdGenerator methodsFor: 'initialize-release' stamp: 'klub 5/21/2015 19:51'!
initialize

	counter := -1.
	limit := 52.
	length := 1! !

!StreamingHtmlIdGenerator methodsFor: 'accessing' stamp: 'klub 5/21/2015 19:54'!
nextId

	| remainder result |
	(counter := counter + 1) = limit ifTrue: [
		limit := limit * 52.
		length := length + 1.
		counter := 0 ].
	remainder := counter.
	result := ByteString new: length.
	length to: 1 by: -1 do: [ :index |
		result at: index put: ('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' at: remainder \\ 52 + 1).
		remainder := remainder // 52 ].
	^result! !

Object subclass: #StreamingHtmlIdGeneratorWithBaseId
	instanceVariableNames: 'counter baseId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Core'!

!StreamingHtmlIdGeneratorWithBaseId methodsFor: 'accessing' stamp: 'klub 4/20/2015 15:33'!
baseId: aByteString

	baseId := aByteString! !

!StreamingHtmlIdGeneratorWithBaseId methodsFor: 'initialize-release' stamp: 'klub 4/20/2015 15:33'!
initializeBaseId

	baseId := SmallInteger maxVal atRandom printStringBase: 36 length: 6 padded: true! !

!StreamingHtmlIdGeneratorWithBaseId methodsFor: 'accessing' stamp: 'klub 4/20/2015 15:34'!
nextId

	baseId ifNil: [ self initializeBaseId ].
	^String new: baseId size + 3 streamContents: [ :stream |
		(counter := counter + 1) printOn: stream base: 36 length: 3 padded: true.
		stream nextPutAll: baseId ]! !

Object subclass: #StreamingHtmlScriptCollector
	instanceVariableNames: 'stream'
	classVariableNames: 'CharactersToEncode'
	poolDictionaries: ''
	category: 'StreamingHtmlCanvas-Core'!

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'klub 11/18/2012 11:08'!
call: functionName withArguments: arguments

	stream 
		nextPutAll: functionName;
		nextPut: $(.
	arguments
		do: [ :each | each jsonWriteOn: stream ]
		separatedBy: [ stream nextPut: $, ].
	stream nextPutAll: ');'! !

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'ul 11/1/2016 18:08'!
createScopeFor: aBlock

	stream nextPutAll: '(function(){'.
	aBlock value.
	stream nextPutAll: '})();'! !

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'klub 11/18/2012 11:05'!
initialize

	stream := (String new: 100) writeStream! !

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'klub 11/18/2012 11:06'!
nextPutAll: aString

	stream nextPutAll: aString! !

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'ul 7/30/2018 15:06'!
renderOn: html

	html script: [ html encodeScript: stream contents ]! !

!StreamingHtmlScriptCollector methodsFor: 'as yet unclassified' stamp: 'klub 11/18/2012 11:13'!
whenReadyCall: functionName withArguments: arguments

	stream nextPutAll: '$(function(){'.
	self call: functionName withArguments: arguments.
	stream nextPutAll: '});'! !

!BlockClosure methodsFor: '*StreamingHtmlCanvas' stamp: 'ul 11/1/2011 15:50'!
renderOn: html

	self cull: html! !
StreamingHtmlCanvas initialize!
